/** \file canmon.c


   CAN bus monitor for the SJA1000
 
   \author Cristiano Brudna
   \date 2001, 2002, 2003 

   University of Ulm, Germany
 

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <sys/ioctl.h>   

#include "can.h"

#include "canlib.h"

int can, done = 0;
int plot = 0;

/* text format */
char *BLACK       = "\033[30m",
  *RED         = "\033[31m",
  *GREEN       = "\033[32m",
  *YELLOW      = "\033[33m",
  *BLUE        = "\033[34m",
  *MAGENTA     = "\033[35m",
  *CYAN        = "\033[36m",
  *WHITE       = "\033[37m",
  *RESET       = "\033[m",
  *BOLD        = "\033[1m",
  *ITALIC      = "\033[3m",
  *UNDERLINE   = "\033[4m",
  *INVERSE     = "\033[7m",
  *CROSSEDPOUT = "\033[9m";

void siginthandler(int a)
{
  if(!plot)
    showStatus(can);
  done = 1;
  
}


void monitor(void)
{
  int n = 0, i, ret;
  long long int time, start_time=-1;
  canmsg msg;
  
  if(!plot)
    printf("type -> identifier [ data ]  timestamp \n\n");
  
  while(!done) {
    ret = read(can, &msg, sizeof(canmsg));
    if(ret > 0) {
      n++;
      if(!plot) {
        printf(BOLD);
        if(msg.type == EXTENDED)
          printf("Extended");
        else
          printf("Standard");

        if(msg.rtr == 1)
          printf(" remote");
        else
          printf(" data");

        printf(" frame -> "); 
        printf(RESET);
        printf(CYAN);
        printf("0x%x  ", msg.id);
        printf(RESET);
        printf("[");

        printf(GREEN);
        for(i=0; i<msg.len; i++)
          printf(" 0x%02x", msg.d[i]);
        printf(RESET);
        printf(" ]  ");
        printf("%lds %06ldµs \n\n", msg.timestamp.tv_sec, msg.timestamp.tv_usec);

      } else {
        time = msg.timestamp.tv_sec * 1000000 + msg.timestamp.tv_usec;
        if(start_time == -1)
          start_time = time;
        printf("%Ld 1\n", (time - start_time));
        
      }
    }
    fflush(stdout);
  }
}

int main(int argc, char *argv[])
{
  unsigned long baud_rate=B1000;
  long long acode=0xFFFFFFFF, amask=0xFFFFFFFF;
  char opchr;
  int par;
  

  if(!plot)
    printf("\nCAN Monitor \n\n");
  
  can = open("/dev/can", O_RDWR);
  
  ioctl(can, CAN_IOCSBAUD, &baud_rate);
  
  ioctl(can, CAN_IOCSACODE, &acode);
 
  ioctl(can, CAN_IOCSAMASK, &amask);

  
  if(!plot)
    showStatus(can);
  
  signal(SIGINT, siginthandler);
  siginterrupt(SIGINT, 1);
  
  monitor();
  close(can);
  exit(0);
}
