/** \file cansend.c

   CAN bus monitor for the SJA1000

   \author Cristiano Brudna
   \date 2001, 2002, 2003
   
   University of Ulm, Germany
   
   
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "can.h"

#include "canlib.h"


int can;


void usage(void)
{
  fprintf(stderr,"\nusage:  cansend [-b baudrate] <#msgs> [message]\n\n");
  fprintf(stderr," message = <type> <id> (<data>) {0,8}\n");
  fprintf(stderr,"           type is 's' for standard and 'e' for extended frame \n");
  fprintf(stderr,"           id and data must be in hexadecimal \n");
  fprintf(stderr, " -b baud:  set baudrate [1000, 500, 250, 125, 20] kbit/s\n");
}


int main(int argc, char *argv[])
{
  char type;
  int num, k, dat, count, i, j, err;
  unsigned long baud_rate=B1000;
  canmsg txmsg;
  int par, pos = 0;  /* pos is 2 if there is a baud rate request, and 0 if not */


  can = open("/dev/can", O_RDWR);
  
  showStatus(can);
  ioctl(can, CAN_IOCSBAUD, &baud_rate);
  printf("===========\n");

  txmsg.type = EXTENDED;

  txmsg.id =0x12;

  txmsg.rtr = 0;

  txmsg.len = 8;

  for (i=0;i<8;i++) 
  txmsg.d[i] =i;

  printf("sending message %d times to /dev/can\n\n", count);
  
  printf("identifier < priority txnode etag > [ data ] timestamp \n\n");
  
  fflush(stdout);
  
  for(i=0; i<count; i++) {
    num = write(can, &txmsg, sizeof(txmsg));
    if(num<0)
      printf("error while writing\n");
    usleep(100);
    
    err = ioctl(can, CAN_IOCRTTS, &txmsg.timestamp);
    printf("Frame -> %x [", txmsg.id);
    for(j=0; j<txmsg.len; j++)
      printf(" %x", txmsg.d[j]);
    printf(" ] %lds %ldµs \n", txmsg.timestamp.tv_sec, txmsg.timestamp.tv_usec);
 
   }
  
  printf("\n");
  
  /* wait for transmission */
  usleep(500000);
  
  showStatus(can);
  close(can);
  exit(0);
}
